<?php

declare(strict_types=1);

namespace Drupal\group_domain\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Site\Settings;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group_domain\GroupDomainInfo;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Path subscriber for controller requests.
 */
final class PathRequestSubscriber implements EventSubscriberInterface {

  /**
   * The ETM.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The RM.
   */
  private RouteMatchInterface $routeMatch;

  /**
   * The domain info service.
   */
  private GroupDomainInfo $domainInfo;

  /**
   * The main domain.
   */
  private ?string $mainDomain;

  /**
   * Constructor.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    ConfigFactoryInterface $config_factory,
    GroupDomainInfo $domain_info
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $route_match;
    $this->domainInfo = $domain_info;

    $config = $config_factory->get(GroupDomainInfo::CONFIG_NAME);
    $this->mainDomain = $config->get('main_domain');
  }

  /**
   * Handle response.
   *
   * Throw HTTP not found if request is for content which is not nested under
   * the right custom domain.
   */
  public function onResponse(ResponseEvent $event): void {
    if (!$event->isMainRequest()) {
      return;
    }

    // @todo Remove that killswitch if it doesn't serve any purpose.
    if (Settings::get('group_domain.custom_domain_kill', FALSE) !== FALSE) {
      @\trigger_error("Using killswitch is deprecated in 1.0.x and shouldn't be relied on unless a proper use case is found.", \E_USER_DEPRECATED);
      return;
    }

    $request = $event->getRequest();

    // Don't act on the main domain as well, all should be accessible there.
    if ($this->mainDomain !== NULL && $request->getHttpHost() === $this->mainDomain) {
      return;
    }

    $group = $this->routeMatch->getParameter('group');
    $group_content = $this->routeMatch->getParameter('group_content');
    if ($group instanceof GroupInterface) {
      $this->throwIfNotGroupNestedContent($group, $request);
    }
    elseif ($group_content instanceof GroupContentInterface) {
      $this->throwIfNotGroupNestedContent($group_content->getGroup(), $request);
    }
  }

  /**
   * Throw exception.
   *
   * Throw if request is for content which is not nested under the right
   * custom domain.
   */
  private function throwIfNotGroupNestedContent(GroupInterface $group, Request $request): void {
    $domain_group = $this->domainInfo->getRequestDomainGroup($request);
    if ($domain_group === NULL || $domain_group->id() !== $group->id()) {
      throw new NotFoundHttpException();
    }
  }

  /**
   * @return array<string, mixed[]>
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[KernelEvents::RESPONSE][] = ['onResponse'];
    return $events;
  }

}
